var FormHandler = (function (j) {

	/*** FUNZIONI DI UTILITA' ***/
	function FormHandler__addMessage(type, dismittable, msgKey) {
		var htmlString = "<div class=\"alert alert-"+type+"\">";
		if(dismittable) {
			var dismitBtn = "<button type=\"button\" class=\"close\" data-dismiss=\"alert\" aria-hidden=\"true\">&times;</button>";
			htmlString = htmlString + dismitBtn;
		}
		var msgTemplate = j('#'+msgKey);
		var msgText = msgTemplate.html();
		htmlString = htmlString + msgText + "</div>";
		jQuery("#msges").append(htmlString);
	}
	
	function FormHandler__collectParametersOneStep( obj, prefix, el) {
		if(typeof prefix != "string") {
			throw new Error("prefix must be a string");
		}
		
		var prefixLength = prefix.length;
		
		j.each(j(el).get(0).attributes, function(index, attrNode) {
			if(this.specified) {
				var attrName = this.name;
				if(attrName.indexOf(prefix)==0) {
					var paramName = attrName.substring(prefixLength);
					
					if(!obj.hasOwnProperty(paramName)) {
						obj[paramName] = this.value;
					}
				}
			}
			
		});
	};
	
	function FormHandler__collectParameters(evt) {
		var result = {};
		var delegateTarget = j(evt.delegateTarget);
		var menuName = delegateTarget.attr('data-menu-name'); 
		
		var prefix = "data-"+menuName+"-";
		
		var elementNow = j(evt.target);
		while( elementNow) {
			
			FormHandler__collectParametersOneStep(result, prefix, elementNow);
			
			if(elementNow.get(0) == delegateTarget.get(0)) {
				elementNow = null;
			} else {
				elementNow = elementNow.parent();
			}
		}
		return result;
	}
	
	
	
	/*** FUNZIONE COSTRUTTORE DELLA CLASSE PER LA GESTIONE DEI FORM ***/
	/*** ogni istanza gestisce un form ***/
	function FormHandler(options) {
		
		var model = (options.model) ? options.model : {};
		var info = (options.info) ? options.info : {};
		
		var _private = {
			selector : options.selector,
			actions: options.actions,
			noAction: options.defaultAction,
			model: model,
			info: info
		};
		
		this.getModel = function FormHandler__getModel() {
			return _private.model;
		}
		
		this.getInfo = function FormHandler__getInfo() {
			return _private.info;
		}
		
		this.init = function FormHandler__init() {
			var self = this;
			
			var jQueryElements = j(_private.selector);
			
			jQueryElements.on('click', function(evt) {
				var registrationEl = this;
				var parameters = FormHandler__collectParameters(evt);
				
				var actionFn = null;
				var actionName = parameters.action;
				if(actionName) {
					actionFn = _private.actions[actionName];
				}
				
				if(!actionFn) {
					actionFn = _private.noAction;
				}
				
				if(actionFn) {
					actionFn(self, parameters, registrationEl, evt);
				} else {
					throw new Error("action \""+actionName+"\" not found");
				}
			});
			
			this.draw();
			
		}
	
		function FormHandler__drawOneTemplate(jEl) {
			var templateId = jEl.attr('data-template');
			
			var params = {};
			FormHandler__collectParametersOneStep( params, 'data-template-', jEl)
			var o = { 
				parameters: params,
				model : _private.model
			};
			
			var templateHtmlResult = tmpl(templateId, o);
			jEl.html(templateHtmlResult);
		}
		
		this.draw = function FormHandler__draw() {
			
			var jQueryElements = j(_private.selector);
			
			
			jQueryElements.each(function(idx, el) {
				var jRootEl = j(el);
				if(jRootEl.is('*[data-template]')) {
					FormHandler__drawOneTemplate(jRootEl);
				} else {
					jRootEl.find('*[data-template]').each( function(idx, el) {
						var jEl = j(el);
						FormHandler__drawOneTemplate(jEl);
					});
				}
			});
			
		}
		
		this.addMessage = FormHandler__addMessage;
	
	}
	
return FormHandler;
})(jQuery);
